@@ -1,8 +1,8 @@
 /* unique.c - attribute uniqueness module */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/unique.c,v 1.20.2.5 2008/02/11 23:26:49 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/unique.c,v 1.20.2.16 2009/08/02 18:44:04 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2004-2008 The OpenLDAP Foundation.
+ * Copyright 2004-2009 The OpenLDAP Foundation.
  * Portions Copyright 2004,2006-2007 Symas Corporation.
  * All rights reserved.
  *
@@ -44,16 +44,17 @@ typedef struct unique_attrs_s {
 
 typedef struct unique_domain_uri_s {
 	struct unique_domain_uri_s *next;
-	struct berval *dn;
-	struct berval *ndn;
-	struct berval *filter;
+	struct berval dn;
+	struct berval ndn;
+	struct berval filter;
+	Filter *f;
 	struct unique_attrs_s *attrs;
 	int scope;
 } unique_domain_uri;
 
 typedef struct unique_domain_s {
 	struct unique_domain_s *next;
-	struct berval *domain_spec;
+	struct berval domain_spec;
 	struct unique_domain_uri_s *uri;
 	char ignore;                          /* polarity of attributes */
 	char strict;                          /* null considered unique too */
@@ -138,9 +139,10 @@ unique_free_domain_uri ( unique_domain_uri *uri )
 
 	while ( uri ) {
 		next_uri = uri->next;
-		ber_bvfree ( uri->dn );
-		ber_bvfree ( uri->ndn );
-		ber_bvfree ( uri->filter );
+		ch_free ( uri->dn.bv_val );
+		ch_free ( uri->ndn.bv_val );
+		ch_free ( uri->filter.bv_val );
+		filter_free( uri->f );
 		attr = uri->attrs;
 		while ( attr ) {
 			next_attr = attr->next;
@@ -160,7 +162,7 @@ unique_free_domain ( unique_domain *domain )
 
 	while ( domain ) {
 		next_domain = domain->next;
-		ber_bvfree ( domain->domain_spec );
+		ch_free ( domain->domain_spec.bv_val );
 		unique_free_domain_uri ( domain->uri );
 		ch_free ( domain );
 		domain = next_domain;
@@ -183,11 +185,11 @@ unique_new_domain_uri ( unique_domain_uri **urip,
 	uri = ch_calloc ( 1, sizeof ( unique_domain_uri ) );
 
 	if ( url_desc->lud_dn && url_desc->lud_dn[0] ) {
-		ber_str2bv( url_desc->lud_dn, 0, 1, &bv );
+		ber_str2bv( url_desc->lud_dn, 0, 0, &bv );
 		rc = dnPrettyNormal( NULL,
 				     &bv,
-				     uri->dn,
-				     uri->ndn,
+				     &uri->dn,
+				     &uri->ndn,
 				     NULL );
 		if ( rc != LDAP_SUCCESS ) {
 			snprintf( c->cr_msg, sizeof( c->cr_msg ),
@@ -197,14 +199,30 @@ unique_new_domain_uri ( unique_domain_uri **urip,
 			goto exit;
 		}
 
-		if ( !dnIsSuffix ( uri->ndn, &be->be_nsuffix[0] ) ) {
+		if ( be->be_nsuffix == NULL ) {
+			snprintf( c->cr_msg, sizeof( c->cr_msg ),
+				  "suffix must be set" );
+			Debug ( LDAP_DEBUG_CONFIG, "unique config: %s\n",
+				c->cr_msg, NULL, NULL );
+			rc = ARG_BAD_CONF;
+			goto exit;
+		}
+
+		if ( !dnIsSuffix ( &uri->ndn, &be->be_nsuffix[0] ) ) {
 			snprintf( c->cr_msg, sizeof( c->cr_msg ),
 				  "dn <%s> is not a suffix of backend base dn <%s>",
-				  uri->dn->bv_val,
+				  uri->dn.bv_val,
 				  be->be_nsuffix[0].bv_val );
 			rc = ARG_BAD_CONF;
 			goto exit;
 		}
+
+		if ( BER_BVISNULL( &be->be_rootndn ) || BER_BVISEMPTY( &be->be_rootndn ) ) {
+			Debug( LDAP_DEBUG_ANY,
+				"slapo-unique needs a rootdn; "
+				"backend <%s> has none, YMMV.\n",
+				be->be_nsuffix[0].bv_val, 0, 0 );
+		}
 	}
 
 	attr_str = url_desc->lud_attrs;
@@ -238,19 +256,26 @@ unique_new_domain_uri ( unique_domain_uri **urip,
 	}
 
 	if (url_desc->lud_filter) {
-		Filter * f;
-		uri->filter = ber_str2bv( url_desc->lud_filter, 0, 1, NULL);
-		f = str2filter( uri->filter->bv_val );
-		if ( !f ) {
+		char *ptr;
+		uri->f = str2filter( url_desc->lud_filter );
+		if ( !uri->f ) {
+			snprintf( c->cr_msg, sizeof( c->cr_msg ),
+				  "unique: bad filter");
+			rc = ARG_BAD_CONF;
+			goto exit;
+		}
+		/* make sure the strfilter is in normal form (ITS#5581) */
+		filter2bv( uri->f, &uri->filter );
+		ptr = strstr( uri->filter.bv_val, "(?=" /*)*/ );
+		if ( ptr != NULL && ptr <= ( uri->filter.bv_val - STRLENOF( "(?=" /*)*/ ) + uri->filter.bv_len ) )
+		{
 			snprintf( c->cr_msg, sizeof( c->cr_msg ),
 				  "unique: bad filter");
 			rc = ARG_BAD_CONF;
 			goto exit;
 		}
-		filter_free( f );
 	}
 exit:
-	if ( bv.bv_val ) ber_memfree ( bv.bv_val );
 	uri->next = *urip;
 	*urip = uri;
 	if ( rc ) {
@@ -305,7 +330,7 @@ unique_new_domain ( unique_domain **domainp,
 	      domain_spec, 0, 0);
 
 	domain = ch_calloc ( 1, sizeof (unique_domain) );
-	domain->domain_spec = ber_str2bv( domain_spec, 0, 1, NULL );
+	ber_str2bv( domain_spec, 0, 1, &domain->domain_spec );
 
 	uri_start = domain_spec;
 	if ( strncasecmp ( uri_start, "ignore ",
@@ -372,24 +397,23 @@ unique_cf_base( ConfigArgs *c )
 	switch ( c->op ) {
 	case SLAP_CONFIG_EMIT:
 		rc = 0;
-		if ( legacy && legacy->uri && legacy->uri->dn ) {
+		if ( legacy && legacy->uri && legacy->uri->dn.bv_val ) {
 			rc = value_add_one ( &c->rvalue_vals,
-					     legacy->uri->dn );
+					     &legacy->uri->dn );
 			if ( rc ) return rc;
 			rc = value_add_one ( &c->rvalue_nvals,
-					     legacy->uri->ndn );
+					     &legacy->uri->ndn );
 			if ( rc ) return rc;
 		}
 		break;
 	case LDAP_MOD_DELETE:
-		assert ( legacy && legacy->uri && legacy->uri->dn );
+		assert ( legacy && legacy->uri && legacy->uri->dn.bv_val );
 		rc = 0;
-		ber_bvfree ( legacy->uri->dn );
-		ber_bvfree ( legacy->uri->ndn );
-		legacy->uri->dn = NULL;
-		legacy->uri->ndn = NULL;
-		if ( !legacy->uri->attrs
-		     && !legacy->uri->dn ) {
+		ch_free ( legacy->uri->dn.bv_val );
+		ch_free ( legacy->uri->ndn.bv_val );
+		BER_BVZERO( &legacy->uri->dn );
+		BER_BVZERO( &legacy->uri->ndn );
+		if ( !legacy->uri->attrs ) {
 			unique_free_domain_uri ( legacy->uri );
 			legacy->uri = NULL;
 		}
@@ -408,6 +432,14 @@ unique_cf_base( ConfigArgs *c )
 			rc = ARG_BAD_CONF;
 			break;
 		}
+		if ( be->be_nsuffix == NULL ) {
+			snprintf( c->cr_msg, sizeof( c->cr_msg ),
+				  "suffix must be set" );
+			Debug ( LDAP_DEBUG_CONFIG, "unique config: %s\n",
+				c->cr_msg, NULL, NULL );
+			rc = ARG_BAD_CONF;
+			break;
+		}
 		if ( !dnIsSuffix ( &c->value_ndn,
 				   &be->be_nsuffix[0] ) ) {
 			snprintf( c->cr_msg, sizeof( c->cr_msg ),
@@ -425,16 +457,23 @@ unique_cf_base( ConfigArgs *c )
 		}
 		if ( !legacy->uri )
 			unique_new_domain_uri_basic ( &legacy->uri, c );
-		ber_bvfree ( legacy->uri->dn );
-		ber_bvfree ( legacy->uri->ndn );
-		legacy->uri->dn = ber_bvdup ( &c->value_dn );
-		legacy->uri->ndn = ber_bvdup ( &c->value_ndn );
+		ch_free ( legacy->uri->dn.bv_val );
+		ch_free ( legacy->uri->ndn.bv_val );
+		legacy->uri->dn = c->value_dn;
+		legacy->uri->ndn = c->value_ndn;
 		rc = 0;
 		break;
 	default:
 		abort();
 	}
 
+	if ( rc ) {
+		ch_free( c->value_dn.bv_val );
+		BER_BVZERO( &c->value_dn );
+		ch_free( c->value_ndn.bv_val );
+		BER_BVZERO( &c->value_ndn );
+	}
+
 	return rc;
 }
 
@@ -485,7 +524,7 @@ unique_cf_attrs( ConfigArgs *c )
 				ch_free (attr);
 			}
 			if ( !legacy->uri->attrs
-			     && !legacy->uri->dn ) {
+			     && !legacy->uri->dn.bv_val ) {
 				unique_free_domain_uri ( legacy->uri );
 				legacy->uri = NULL;
 			}
@@ -663,10 +702,7 @@ unique_cf_uri( ConfigArgs *c )
 		      domain;
 		      domain = domain->next ) {
 			rc = value_add_one ( &c->rvalue_vals,
-					     domain->domain_spec );
-			if ( rc ) break;
-			rc = value_add_one ( &c->rvalue_nvals,
-					     domain->domain_spec );
+					     &domain->domain_spec );
 			if ( rc ) break;
 		}
 		break;
@@ -952,7 +988,7 @@ unique_search(
 	struct berval * dn,
 	int scope,
 	SlapReply *rs,
-	char *key
+	struct berval *key
 )
 {
 	slap_overinst *on = (slap_overinst *) op->o_bd->bd_info;
@@ -961,10 +997,17 @@ unique_search(
 	unique_counter uq = { NULL, 0 };
 	int rc;
 
-	Debug(LDAP_DEBUG_TRACE, "==> unique_search %s\n", key, 0, 0);
+	Debug(LDAP_DEBUG_TRACE, "==> unique_search %s\n", key->bv_val, 0, 0);
 
-	nop->ors_filter = str2filter_x(nop, key);
-	ber_str2bv(key, 0, 0, &nop->ors_filterstr);
+	nop->ors_filter = str2filter_x(nop, key->bv_val);
+	if(nop->ors_filter == NULL) {
+		op->o_bd->bd_info = (BackendInfo *) on->on_info;
+		send_ldap_error(op, rs, LDAP_OTHER,
+			"unique_search invalid filter");
+		return(rs->sr_err);
+	}
+
+	nop->ors_filterstr = *key;
 
 	cb.sc_response	= (slap_response*)count_attr_cb;
 	cb.sc_private	= &uq;
@@ -985,8 +1028,8 @@ unique_search(
 
 	nop->o_bd = on->on_info->oi_origdb;
 	rc = nop->o_bd->be_search(nop, &nrs);
-	filter_free_x(nop, nop->ors_filter);
-	op->o_tmpfree( key, op->o_tmpmemctx );
+	filter_free_x(nop, nop->ors_filter, 1);
+	op->o_tmpfree( key->bv_val, op->o_tmpmemctx );
 
 	if(rc != LDAP_SUCCESS && rc != LDAP_NO_SUCH_OBJECT) {
 		op->o_bd->bd_info = (BackendInfo *) on->on_info;
@@ -1020,6 +1063,7 @@ unique_add(
 	Operation nop = *op;
 	Attribute *a;
 	char *key, *kp;
+	struct berval bvkey;
 	int rc = SLAP_CB_CONTINUE;
 
 	Debug(LDAP_DEBUG_TRACE, "==> unique_add <%s>\n",
@@ -1030,7 +1074,7 @@ unique_add(
 	      domain = domain->next )
 	{
 		unique_domain_uri *uri;
-		int ks = 0;
+		int ks = STRLENOF("(|)");
 
 		for ( uri = domain->uri;
 		      uri;
@@ -1038,10 +1082,21 @@ unique_add(
 		{
 			int len;
 
-			if ( uri->ndn
-			     && !dnIsSuffix( &op->o_req_ndn, uri->ndn ))
+			if ( uri->ndn.bv_val
+			     && !dnIsSuffix( &op->o_req_ndn, &uri->ndn ))
 				continue;
 
+			if ( uri->f ) {
+				if ( test_filter( NULL, op->ora_e, uri->f )
+					== LDAP_COMPARE_FALSE )
+				{
+					Debug( LDAP_DEBUG_TRACE,
+						"==> unique_add_skip<%s>\n",
+						op->o_req_dn.bv_val, 0, 0 );
+					continue;
+				}
+			}
+
 			if(!(a = op->ora_e->e_attrs)) {
 				op->o_bd->bd_info = (BackendInfo *) on->on_info;
 				send_ldap_error(op, rs, LDAP_INVALID_SYNTAX,
@@ -1061,12 +1116,15 @@ unique_add(
 			/* skip this domain-uri if it isn't involved */
 			if ( !ks ) continue;
 
-			if ( uri->filter && uri->filter->bv_len )
-				ks += uri->filter->bv_len + STRLENOF ("(&)");
+			/* terminating NUL */
+			ks++;
+
+			if ( uri->filter.bv_val && uri->filter.bv_len )
+				ks += uri->filter.bv_len + STRLENOF ("(&)");
 			kp = key = op->o_tmpalloc(ks, op->o_tmpmemctx);
 
-			if ( uri->filter && uri->filter->bv_len ) {
-				len = snprintf (kp, ks, "(&%s", uri->filter->bv_val);
+			if ( uri->filter.bv_val && uri->filter.bv_len ) {
+				len = snprintf (kp, ks, "(&%s", uri->filter.bv_val);
 				assert( len >= 0 && len < ks );
 				kp += len;
 			}
@@ -1086,20 +1144,22 @@ unique_add(
 			len = snprintf(kp, ks - (kp - key), ")");
 			assert( len >= 0 && len < ks - (kp - key) );
 			kp += len;
-			if ( uri->filter && uri->filter->bv_len ) {
+			if ( uri->filter.bv_val && uri->filter.bv_len ) {
 				len = snprintf(kp, ks - (kp - key), ")");
 				assert( len >= 0 && len < ks - (kp - key) );
 				kp += len;
 			}
+			bvkey.bv_val = key;
+			bvkey.bv_len = kp - key;
 
 			rc = unique_search ( op,
 					     &nop,
-					     uri->ndn ?
-					     uri->ndn :
+					     uri->ndn.bv_val ?
+					     &uri->ndn :
 					     &op->o_bd->be_nsuffix[0],
 					     uri->scope,
 					     rs,
-					     key);
+					     &bvkey);
 
 			if ( rc != SLAP_CB_CONTINUE ) break;
 		}
@@ -1124,6 +1184,7 @@ unique_modify(
 	Operation nop = *op;
 	Modifications *m;
 	char *key, *kp;
+	struct berval bvkey;
 	int rc = SLAP_CB_CONTINUE;
 
 	Debug(LDAP_DEBUG_TRACE, "==> unique_modify <%s>\n",
@@ -1134,7 +1195,7 @@ unique_modify(
 	      domain = domain->next )
 	{
 		unique_domain_uri *uri;
-		int ks = 0;
+		int ks = STRLENOF("(|)");
 
 		for ( uri = domain->uri;
 		      uri;
@@ -1142,8 +1203,8 @@ unique_modify(
 		{
 			int len;
 
-			if ( uri->ndn
-			     && !dnIsSuffix( &op->o_req_ndn, uri->ndn ))
+			if ( uri->ndn.bv_val
+			     && !dnIsSuffix( &op->o_req_ndn, &uri->ndn ))
 				continue;
 
 			if ( !(m = op->orm_modlist) ) {
@@ -1166,12 +1227,15 @@ unique_modify(
 			/* skip this domain-uri if it isn't involved */
 			if ( !ks ) continue;
 
-			if ( uri->filter && uri->filter->bv_len )
-				ks += uri->filter->bv_len + STRLENOF ("(&)");
+			/* terminating NUL */
+			ks++;
+
+			if ( uri->filter.bv_val && uri->filter.bv_len )
+				ks += uri->filter.bv_len + STRLENOF ("(&)");
 			kp = key = op->o_tmpalloc(ks, op->o_tmpmemctx);
 
-			if ( uri->filter && uri->filter->bv_len ) {
-				len = snprintf(kp, ks, "(&%s", uri->filter->bv_val);
+			if ( uri->filter.bv_val && uri->filter.bv_len ) {
+				len = snprintf(kp, ks, "(&%s", uri->filter.bv_val);
 				assert( len >= 0 && len < ks );
 				kp += len;
 			}
@@ -1193,20 +1257,22 @@ unique_modify(
 			len = snprintf(kp, ks - (kp - key), ")");
 			assert( len >= 0 && len < ks - (kp - key) );
 			kp += len;
-			if ( uri->filter && uri->filter->bv_len ) {
+			if ( uri->filter.bv_val && uri->filter.bv_len ) {
 				len = snprintf (kp, ks - (kp - key), ")");
 				assert( len >= 0 && len < ks - (kp - key) );
 				kp += len;
 			}
+			bvkey.bv_val = key;
+			bvkey.bv_len = kp - key;
 
 			rc = unique_search ( op,
 					     &nop,
-					     uri->ndn ?
-					     uri->ndn :
+					     uri->ndn.bv_val ?
+					     &uri->ndn :
 					     &op->o_bd->be_nsuffix[0],
 					     uri->scope,
 					     rs,
-					     key);
+					     &bvkey);
 
 			if ( rc != SLAP_CB_CONTINUE ) break;
 		}
@@ -1230,6 +1296,7 @@ unique_modrdn(
 	unique_domain *domain;
 	Operation nop = *op;
 	char *key, *kp;
+	struct berval bvkey;
 	LDAPRDN	newrdn;
 	struct berval bv[2];
 	int rc = SLAP_CB_CONTINUE;
@@ -1242,7 +1309,7 @@ unique_modrdn(
 	      domain = domain->next )
 	{
 		unique_domain_uri *uri;
-		int ks = 0;
+		int ks = STRLENOF("(|)");
 
 		for ( uri = domain->uri;
 		      uri;
@@ -1250,10 +1317,10 @@ unique_modrdn(
 		{
 			int i, len;
 
-			if ( uri->ndn
-			     && !dnIsSuffix( &op->o_req_ndn, uri->ndn )
+			if ( uri->ndn.bv_val
+			     && !dnIsSuffix( &op->o_req_ndn, &uri->ndn )
 			     && (!op->orr_nnewSup
-				 || !dnIsSuffix( op->orr_nnewSup, uri->ndn )))
+				 || !dnIsSuffix( op->orr_nnewSup, &uri->ndn )))
 				continue;
 
 			if ( ldap_bv2rdn_x ( &op->oq_modrdn.rs_newrdn,
@@ -1296,12 +1363,15 @@ unique_modrdn(
 			/* skip this domain if it isn't involved */
 			if ( !ks ) continue;
 
-			if ( uri->filter && uri->filter->bv_len )
-				ks += uri->filter->bv_len + STRLENOF ("(&)");
+			/* terminating NUL */
+			ks++;
+
+			if ( uri->filter.bv_val && uri->filter.bv_len )
+				ks += uri->filter.bv_len + STRLENOF ("(&)");
 			kp = key = op->o_tmpalloc(ks, op->o_tmpmemctx);
 
-			if ( uri->filter && uri->filter->bv_len ) {
-				len = snprintf(kp, ks, "(&%s", uri->filter->bv_val);
+			if ( uri->filter.bv_val && uri->filter.bv_len ) {
+				len = snprintf(kp, ks, "(&%s", uri->filter.bv_val);
 				assert( len >= 0 && len < ks );
 				kp += len;
 			}
@@ -1323,20 +1393,22 @@ unique_modrdn(
 			len = snprintf(kp, ks - (kp - key), ")");
 			assert( len >= 0 && len < ks - (kp - key) );
 			kp += len;
-			if ( uri->filter && uri->filter->bv_len ) {
+			if ( uri->filter.bv_val && uri->filter.bv_len ) {
 				len = snprintf (kp, ks - (kp - key), ")");
 				assert( len >= 0 && len < ks - (kp - key) );
 				kp += len;
 			}
+			bvkey.bv_val = key;
+			bvkey.bv_len = kp - key;
 
 			rc = unique_search ( op,
 					     &nop,
-					     uri->ndn ?
-					     uri->ndn :
+					     uri->ndn.bv_val ?
+					     &uri->ndn :
 					     &op->o_bd->be_nsuffix[0],
 					     uri->scope,
 					     rs,
-					     key);
+					     &bvkey);
 
 			if ( rc != SLAP_CB_CONTINUE ) break;
 		}
